import sinon from "sinon";
import Hyperwallet from "../Hyperwallet";
import hwSdk from "../hwSdk";
import HyperwalletTransaction from "../HyperwalletTransaction";

describe("Hyperwallet.js", () => {
    const host = "http://localhost/rest/v4/";
    const stakeholderToken = "stk-12345";
    const userToken = "usr-1234";
    const authToken = "token";
    const transferMethodToken = "trm-12345";

    let hyperwallet;

    beforeEach(() => {
        sinon.stub(hwSdk, "getHyperwalletConfiguration");
        sinon.stub(HyperwalletTransaction, "get");
        sinon.stub(HyperwalletTransaction, "post");
        sinon.stub(HyperwalletTransaction, "put");
        hyperwallet = new Hyperwallet();
    });

    afterEach(() => {
        hwSdk.getHyperwalletConfiguration.restore();
        HyperwalletTransaction.get.restore();
        HyperwalletTransaction.post.restore();
        HyperwalletTransaction.put.restore();
    });

    describe("getUser()", () => {
        it("should GET if no error", () => {
            const callback = () => {};

            hyperwallet.getUser(callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.get.should.have.been.calledWith({
                url: `${host}users/${userToken}`,
                authenticationToken: authToken,
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.getUser(callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("updateUser()", () => {
        it("should send PUT if no error", () => {
            const callback = () => {};
            const user = {};

            hyperwallet.updateUser(user, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.put.should.have.been.calledWith({
                url: `${host}users/${userToken}`,
                authenticationToken: authToken,
                data: sinon.match(user),
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.updateUser({}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("uploadUserDocuments()", () => {
        it("should send PUT if no error", () => {
            const callback = () => {};
            const data = {
                data: {},
                files: [],
            };

            hyperwallet.uploadUserDocuments(data, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.put.should.have.been.calledWith({
                url: `${host}users/${userToken}`,
                authenticationToken: authToken,
                data: sinon.match(data.data),
                files: sinon.match(data.files),
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.uploadUserDocuments({}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("createStakeholder()", () => {
        it("should send POST if no error", () => {
            const callback = () => {};
            const stakeholder = {};

            hyperwallet.createStakeholder(stakeholder, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.post.should.have.been.calledWith({
                url: `${host}users/${userToken}/business-stakeholders`,
                authenticationToken: authToken,
                data: stakeholder,
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.createStakeholder({}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("createTransferMethod()", () => {
        it("should send POST if no error", () => {
            const callback = () => {};
            const transferMethod = {};

            hyperwallet.createTransferMethod(transferMethod, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.post.should.have.been.calledWith({
                url: `${host}users/${userToken}/transfer-methods`,
                authenticationToken: authToken,
                data: transferMethod,
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.createTransferMethod({}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("listStakeholders()", () => {
        it("should GET if no error", () => {
            const callback = () => {};

            hyperwallet.listStakeholders(true, undefined, false, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.get.should.have.been.calledWith({
                url: `${host}users/${userToken}/business-stakeholders?sortBy=-createdOn&status=ACTIVATED&isDirector=true&isBusinessContact=false`,
                authenticationToken: authToken,
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.listStakeholders(true, undefined, false, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("createStakeholder()", () => {
        it("should send POST if no error", () => {
            const callback = () => {};
            const stakeholder = {};

            hyperwallet.createStakeholder(stakeholder, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.post.should.have.been.calledWith({
                url: `${host}users/${userToken}/business-stakeholders`,
                authenticationToken: authToken,
                data: stakeholder,
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.createStakeholder({}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("updateStakeholder()", () => {
        it("should send PUT if no error", () => {
            const callback = () => {};
            const stakeholder = {};

            hyperwallet.updateStakeholder(stakeholderToken, stakeholder, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.put.should.have.been.calledWith({
                url: `${host}users/${userToken}/business-stakeholders/${stakeholderToken}`,
                authenticationToken: authToken,
                data: sinon.match(stakeholder),
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.updateStakeholder("any-token", {}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("createUserStakeholderStatusTransition()", () => {
        it("should send POST if no error", () => {
            const callback = () => {};
            const statusTransition = "DISABLED";

            hyperwallet.createUserStakeholderStatusTransition(stakeholderToken,
                statusTransition, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.post.should.have.been.calledWith({
                url: `${host}users/${userToken}/business-stakeholders/${stakeholderToken}/status-transitions`,
                authenticationToken: authToken,
                data: sinon.match({
                    transition: statusTransition,
                }),
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.createUserStakeholderStatusTransition("any-token", {}, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });

    describe("getPartnerAuthorization()", () => {
        it("should send POST if no error", () => {
            const callback = () => {};

            hyperwallet.getPartnerAuthorization(transferMethodToken, callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, undefined, {
                restUri: host,
                walletToken: userToken,
                authenticationToken: authToken,
            });

            HyperwalletTransaction.post.should.have.been.calledWith({
                url: `${host}users/${userToken}/transfer-methods/${transferMethodToken}/partner-authorization`,
                authenticationToken: authToken,
                data: sinon.match({}),
                callback,
            });
        });

        it("should send error to callback", (done) => {
            const error = {};

            const callback = (expectedError) => {
                error.should.be.equal(expectedError);
                done();
            };

            hyperwallet.getPartnerAuthorization("any-token", callback);

            hwSdk.getHyperwalletConfiguration.callArgWith(0, error, undefined);
        });
    });
});
